function _slicedToArray(r, e) { return _arrayWithHoles(r) || _iterableToArrayLimit(r, e) || _unsupportedIterableToArray(r, e) || _nonIterableRest(); }
function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }
function _iterableToArrayLimit(r, l) { var t = null == r ? null : "undefined" != typeof Symbol && r[Symbol.iterator] || r["@@iterator"]; if (null != t) { var e, n, i, u, a = [], f = !0, o = !1; try { if (i = (t = t.call(r)).next, 0 === l) { if (Object(t) !== t) return; f = !1; } else for (; !(f = (e = i.call(t)).done) && (a.push(e.value), a.length !== l); f = !0); } catch (r) { o = !0, n = r; } finally { try { if (!f && null != t["return"] && (u = t["return"](), Object(u) !== u)) return; } finally { if (o) throw n; } } return a; } }
function _arrayWithHoles(r) { if (Array.isArray(r)) return r; }
function _toConsumableArray(r) { return _arrayWithoutHoles(r) || _iterableToArray(r) || _unsupportedIterableToArray(r) || _nonIterableSpread(); }
function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }
function _unsupportedIterableToArray(r, a) { if (r) { if ("string" == typeof r) return _arrayLikeToArray(r, a); var t = {}.toString.call(r).slice(8, -1); return "Object" === t && r.constructor && (t = r.constructor.name), "Map" === t || "Set" === t ? Array.from(r) : "Arguments" === t || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(t) ? _arrayLikeToArray(r, a) : void 0; } }
function _iterableToArray(r) { if ("undefined" != typeof Symbol && null != r[Symbol.iterator] || null != r["@@iterator"]) return Array.from(r); }
function _arrayWithoutHoles(r) { if (Array.isArray(r)) return _arrayLikeToArray(r); }
function _arrayLikeToArray(r, a) { (null == a || a > r.length) && (a = r.length); for (var e = 0, n = Array(a); e < a; e++) n[e] = r[e]; return n; }
import { Lut, remapControlPoints } from "@aics/volume-viewer";
import { findFirstChannelMatch } from "./viewerChannelSettings";
import { LUT_MAX_PERCENTILE, LUT_MIN_PERCENTILE, TFEDITOR_DEFAULT_COLOR, TFEDITOR_MAX_BIN } from "../constants";

// @param {Object[]} controlPoints - array of {x:number, opacity:number, color:string}
// @return {Uint8Array} array of length 256*4 representing the rgba values of the gradient
export function controlPointsToLut(controlPoints) {
  var lut = new Lut().createFromControlPoints(controlPoints);
  return lut;
}

/** Returns a default lookup table based on a min/max percentile of the current volume's data. */
export function getDefaultLut(histogram) {
  var hmin = histogram.findBinOfPercentile(LUT_MIN_PERCENTILE);
  var hmax = histogram.findBinOfPercentile(LUT_MAX_PERCENTILE);
  return new Lut().createFromMinMax(hmin, hmax);
}

/**
 * Parses a single LUT value from a string, where the value is either a number, a percentile, or a median multiplier.
 */
function parseLutValue(value, histogram) {
  // look at first char of string.
  var firstChar = value.charAt(0);
  if (firstChar === "m") {
    // median
    var parsedValue = parseFloat(value.substring(1)) / 100.0;
    return histogram.maxBin * parsedValue;
  } else if (firstChar === "p") {
    // percentile
    var _parsedValue = parseFloat(value.substring(1)) / 100.0;
    return histogram.findBinOfPercentile(_parsedValue);
  } else {
    // plain number
    return parseFloat(value);
  }
}

/**
 * Parses a lookup table (LUT) from a `ViewerChannelSetting` object, where the `lut` field is an
 * array of two alphanumeric strings.
 *
 * @returns a Lut object if the `lut` field is valid; otherwise, returns undefined.
 *
 * Min and max values are determined as following:
 * - Plain numbers are direct intensity values.
 * - `p{n}` represents a percentile, where `n` is a percentile in the [0, 100] range.
 * - `m{n}` represents the median multiplied by `n / 100`.
 * - `autoij` in either the min or max fields will use the "auto" algorithm
 * from ImageJ to select the min AND max.
 *
 * @example
 * ```
 * "0:255"    // min: intensity 0, max: intensity 255.
 * "p50:p90"  // min: 50th percentile, max: 90th percentile.
 * "m1:p75"   // min: median, max: 75th percentile.
 * "autoij:0" // use Auto-IJ to calculate min and max.
 * ```
 */
export function parseLutFromSettings(histogram, initSettings) {
  if (initSettings.lut === undefined || initSettings.lut.length !== 2) {
    return undefined;
  }
  var lutValues;
  if (initSettings.lut[0] === "autoij" || initSettings.lut[1] === "autoij") {
    lutValues = histogram.findAutoIJBins();
  } else {
    lutValues = [parseLutValue(initSettings.lut[0], histogram), parseLutValue(initSettings.lut[1], histogram)];
  }
  return new Lut().createFromMinMax(Math.min(lutValues[0], lutValues[1]), Math.max(lutValues[0], lutValues[1]));
}

/**
 * Initializes the lookup table (LUT) that maps from volume intensity values to color + opacity and applies the LUT to the volume.
 *
 * @param aimg The loaded volume data.
 * @param channelIndex The index of the channel to initialize the LUT for.
 * @param channelSettings The ViewerChannelSettings object that may contain settings for this channel. If relevant
 * settings are not found, a default LUT will be used.
 * @returns an object containing the retrieved ramp control points and "advanced mode" control points.
 *
 * LUT values will be determined using the following rules:
 * - If no `lut` is provided in the `channelSettings`, a default LUT is calculated using min/max percentiles of the data.
 * - Otherwise, `lut` will be parsed as described in `ViewerChannelSettingParams.lut`.
 * - The `controlPoints` and `ramp` fields in the `channelSettings` will be used to override the returned "advanced mode"
 * control points and ramp, respectively.
 *
 * If `controlPointsEnabled` is set to true in the `channelSettings`, the "advanced mode" control points will be applied
 * to the volume; otherwise, the ramp will be applied.
 */
export function initializeLut(aimg, channelIndex, channelSettings) {
  var _initSettings$control;
  var histogram = aimg.getHistogram(channelIndex);
  var defaultLut = getDefaultLut(histogram);
  var ramp = [];
  var controlPoints = [];
  var lut = defaultLut;
  var name = aimg.channelNames[channelIndex];
  var initSettings = channelSettings && findFirstChannelMatch(name, channelIndex, channelSettings);

  // Attempt to load a LUT from the settings, which will be used to initialize the control points and ramp
  if (initSettings && initSettings.lut) {
    var _parseLutFromSettings;
    lut = (_parseLutFromSettings = parseLutFromSettings(histogram, initSettings)) !== null && _parseLutFromSettings !== void 0 ? _parseLutFromSettings : defaultLut;
  }
  // Initialize the control points + ramp using the LUT.
  // Optionally, override the LUT's control points with the provided control points and/or ramp.
  controlPoints = (_initSettings$control = initSettings === null || initSettings === void 0 ? void 0 : initSettings.controlPoints) !== null && _initSettings$control !== void 0 ? _initSettings$control : _toConsumableArray(lut.controlPoints);
  ramp = initSettings !== null && initSettings !== void 0 && initSettings.ramp ? rampToControlPoints(initSettings.ramp) : _toConsumableArray(lut.controlPoints);

  // Apply whatever lut is currently visible
  var visibleLut;
  if (initSettings !== null && initSettings !== void 0 && initSettings.controlPointsEnabled) {
    visibleLut = new Lut().createFromControlPoints(controlPoints);
  } else {
    visibleLut = new Lut().createFromControlPoints(ramp);
  }
  aimg.setLut(channelIndex, visibleLut);
  return {
    ramp: ramp,
    controlPoints: controlPoints
  };
}
export function controlPointsToRamp(controlPoints) {
  if (controlPoints.length <= 1) {
    return [0, TFEDITOR_MAX_BIN];
  } else if (controlPoints.length === 2) {
    return [controlPoints[0].x, controlPoints[1].x];
  } else if (controlPoints.length === 3) {
    if (controlPoints[0].opacity !== controlPoints[1].opacity && controlPoints[0].opacity !== controlPoints[2].opacity && controlPoints[1].opacity !== controlPoints[2].opacity) {
      // if all 3 are unequal, assume a ramp from first to last
      return [controlPoints[0].x, controlPoints[2].x];
    } else if (controlPoints[0].opacity !== controlPoints[1].opacity) {
      return [controlPoints[0].x, controlPoints[1].x];
    } else if (controlPoints[1].opacity !== controlPoints[2].opacity) {
      return [controlPoints[1].x, controlPoints[2].x];
    }
  }
  return [controlPoints[1].x, controlPoints[controlPoints.length - 2].x];
}
export function rampToControlPoints(_ref) {
  var _ref2 = _slicedToArray(_ref, 2),
    min = _ref2[0],
    max = _ref2[1];
  return [{
    x: Math.min(min - 1, 0),
    opacity: 0,
    color: TFEDITOR_DEFAULT_COLOR
  }, {
    x: min,
    opacity: 0,
    color: TFEDITOR_DEFAULT_COLOR
  }, {
    x: max,
    opacity: 1,
    color: TFEDITOR_DEFAULT_COLOR
  }, {
    x: Math.max(max + 1, TFEDITOR_MAX_BIN),
    opacity: 1,
    color: TFEDITOR_DEFAULT_COLOR
  }];
}

/** Remaps an array of control points from an old range (as a 2-tuple) to a new one (extracted from a `Channel`) */
export function remapControlPointsForChannel(controlPoints, oldRange, _ref3) {
  var rawMin = _ref3.rawMin,
    rawMax = _ref3.rawMax;
  if (oldRange === undefined) {
    return controlPoints;
  }
  return remapControlPoints(controlPoints, oldRange[0], oldRange[1], rawMin, rawMax);
}