import React from "react";
import type { ViewerState, ViewerStateContextType } from "./types";
export declare const ALL_VIEWER_STATE_KEYS: (keyof ViewerState | "changeViewerSetting" | "setChannelSettings" | "channelSettings" | "changeChannelSetting" | "applyColorPresets" | keyof import("./types").ResetState)[];
type NoNull<T> = {
    [K in keyof T]: NonNullable<T[K]>;
};
type ContextRefType = NoNull<React.MutableRefObject<ViewerStateContextType>>;
export declare const ViewerStateContext: React.Context<{
    ref: ContextRefType;
}>;
/** Provides a central store for the state of the viewer, and the methods to update it. */
declare const ViewerStateProvider: React.FC<{
    viewerSettings?: Partial<ViewerState>;
}>;
/**
 * Higher-order component that connects a component to the viewer state context.
 *
 * Accepts a `component` and an array of `keys` from the viewer state context, and returns a new memoized component
 * which "subscribes" to only those fields of state, and will not re-render if other fields change. This works by
 * creating a component which is "split in two" as described
 * [here](https://react.dev/reference/react/memo#updating-a-memoized-component-using-a-context).
 *
 * NOTE that while higher-order components don't seem to be explicitly considered an anti-pattern, they don't appear in
 * modern documentation because they are "[not commonly used in modern React
 * code](https://legacy.reactjs.org/docs/higher-order-components.html)." React-redux, which inspired this "connect"
 * pattern, has also generally moved away from this pattern [in favor of a hooks-based
 * approach](https://react-redux.js.org/api/hooks). I too would love to use hooks to connect to viewer state, but there
 * is currently no way to hook into context without agreeing to re-render on every change. I feel okay using a higher-
 * order component in this case because, analogous to a custom hook, it extends a provided primitive HOC (`memo`).
 */
export declare function connectToViewerState<Keys extends keyof ViewerStateContextType, Props extends Pick<ViewerStateContextType, Keys>>(component: React.ComponentType<Props>, keys: Keys[]): React.FC<Omit<Props, Keys>>;
export default ViewerStateProvider;
