import { Channel, ControlPoint, Histogram, Lut, Volume } from "@aics/volume-viewer";
import { ViewerChannelSetting, ViewerChannelSettings } from "./viewerChannelSettings";
export declare function controlPointsToLut(controlPoints: ControlPoint[]): Lut;
/** Returns a default lookup table based on a min/max percentile of the current volume's data. */
export declare function getDefaultLut(histogram: Histogram): Lut;
/**
 * Parses a lookup table (LUT) from a `ViewerChannelSetting` object, where the `lut` field is an
 * array of two alphanumeric strings.
 *
 * @returns a Lut object if the `lut` field is valid; otherwise, returns undefined.
 *
 * Min and max values are determined as following:
 * - Plain numbers are direct intensity values.
 * - `p{n}` represents a percentile, where `n` is a percentile in the [0, 100] range.
 * - `m{n}` represents the median multiplied by `n / 100`.
 * - `autoij` in either the min or max fields will use the "auto" algorithm
 * from ImageJ to select the min AND max.
 *
 * @example
 * ```
 * "0:255"    // min: intensity 0, max: intensity 255.
 * "p50:p90"  // min: 50th percentile, max: 90th percentile.
 * "m1:p75"   // min: median, max: 75th percentile.
 * "autoij:0" // use Auto-IJ to calculate min and max.
 * ```
 */
export declare function parseLutFromSettings(histogram: Histogram, initSettings: ViewerChannelSetting): Lut | undefined;
/**
 * Initializes the lookup table (LUT) that maps from volume intensity values to color + opacity and applies the LUT to the volume.
 *
 * @param aimg The loaded volume data.
 * @param channelIndex The index of the channel to initialize the LUT for.
 * @param channelSettings The ViewerChannelSettings object that may contain settings for this channel. If relevant
 * settings are not found, a default LUT will be used.
 * @returns an object containing the retrieved ramp control points and "advanced mode" control points.
 *
 * LUT values will be determined using the following rules:
 * - If no `lut` is provided in the `channelSettings`, a default LUT is calculated using min/max percentiles of the data.
 * - Otherwise, `lut` will be parsed as described in `ViewerChannelSettingParams.lut`.
 * - The `controlPoints` and `ramp` fields in the `channelSettings` will be used to override the returned "advanced mode"
 * control points and ramp, respectively.
 *
 * If `controlPointsEnabled` is set to true in the `channelSettings`, the "advanced mode" control points will be applied
 * to the volume; otherwise, the ramp will be applied.
 */
export declare function initializeLut(aimg: Volume, channelIndex: number, channelSettings?: ViewerChannelSettings): {
    ramp: ControlPoint[];
    controlPoints: ControlPoint[];
};
export declare function controlPointsToRamp(controlPoints: ControlPoint[]): [number, number];
export declare function rampToControlPoints([min, max]: [number, number]): ControlPoint[];
/** Remaps an array of control points from an old range (as a 2-tuple) to a new one (extracted from a `Channel`) */
export declare function remapControlPointsForChannel(controlPoints: ControlPoint[], oldRange: [number, number] | undefined, { rawMin, rawMax }: Channel): ControlPoint[];
